__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

__version__ = "0.0.14"
AUTHOR = "Vanessa Sochat"
NAME = "contributor_ci"
AUTHOR_EMAIL = "vsoch@users.noreply.github.com"
PACKAGE_URL = "https://github.com/vsoch/contributor-ci"
KEYWORDS = "contributions, contributors, GitHub action, automated"
DESCRIPTION = "automated tools to assess contributions."
LICENSE = "LICENSE"

################################################################################
# Global requirements

INSTALL_REQUIRES = (
    ("Jinja2", {"min_version": None}),
    ("jsonschema", {"min_version": None}),
    ("ruamel.yaml", {"min_version": None}),
    ("llnl-scraper", {"min_version": "0.10.0"}),
)

TESTS_REQUIRES = (("pytest", {"min_version": "4.6.2"}),)

################################################################################
# Submodule Requirements (versions that include database)

INSTALL_REQUIRES_ALL = INSTALL_REQUIRES + TESTS_REQUIRES



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

from contributor_ci.main.extractor import GitHubExtractorBase
from contributor_ci.logger import logger
from scraper.github import queryManager as qm


class Languages(GitHubExtractorBase):

    name = "languages"
    description = "extract languages for a repository."
    depends_on = ["repos"]
    filenames = ["languages"]

    def extract(self):
        """
        Extract metadata about repository languages.
        """
        self._data[self.name] = {}
        lang_query = self.get_local_query("repo-languages.gql")
        repos = self.load_dependency_file("repos")

        # Prepare repository names
        repolist = sorted(repos.data.keys())
        logger.info("Found %s associated repos parsed today." % len(repolist))
        for repo in repolist:
            logger.info("Looking up languages for %s" % repo)
            repo_user, repo_name = repo.split("/")

            try:
                out = self.manager.queryGitHubFromFile(
                    lang_query,
                    {
                        "ownName": repo_user,
                        "repoName": repo_name,
                        "numLangs": 25,
                        "pgCursor": None,
                    },
                    paginate=True,
                    cursorVar="pgCursor",
                    keysToList=["data", "repository", "languages", "nodes"],
                )
            except Exception as error:
                logger.warning("Warning: Could not complete '%s': %s" % (repo, error))
                continue

            self._data[self.name][repo] = out["data"]["repository"]



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

from contributor_ci.main.extractor import GitHubExtractorBase
from contributor_ci.logger import logger
from scraper.github import queryManager as qm


class UserExtractor(GitHubExtractorBase):
    """
    A separate extractor used by CCI init to get metadata for one user or org
    """

    def extract(self, username):

        # username is provided as user:<username> or org:orgname
        usertype, username = username.split(":", 1)
        config = {"member_orgs": None, "orgs": set(), "repos": set()}

        # If no username provided, we use running user account:
        if usertype == "user":
            user_query = self.get_local_query("user-info.gql")

            out = self.manager.queryGitHubFromFile(
                user_query,
                {"numRepos": 100, "pgCursor": None},
                paginate=True,
                cursorVar="pgCursor",
                keysToList=["data", "viewer", "repositories", "nodes"],
            )

            for node in out["data"]["viewer"]["repositories"]["nodes"]:

                # skip private repos
                if node["isPrivate"]:
                    continue

                if node["owner"]["login"] == username:
                    config["repos"].add("%s/%s" % (username, node["name"]))
                else:
                    config["orgs"].add(node["owner"]["login"])

        elif usertype == "org":
            config["member_orgs"] = [username]
            config["orgs"].add(username)
        else:
            logger.exit(
                "%s is not a valid extraction type. Please specify user:<username> or org:<orgname>"
            )

        config["orgs"] = list(config["orgs"])
        config["repos"] = list(config["repos"])
        return config


class Users(GitHubExtractorBase):

    name = "users"
    description = "extract user metrics for a repository."
    depends_on = ["repos"]
    filenames = ["internal-users", "external-users"]

    def extract(self):
        """
        Extract metadata about internal and exernal users.
        """
        self.extract_internal_users()
        self.extract_external_users()

    def extract_internal_users(self):
        """
        Extract internal users from member orgs
        """
        self._data["internal-users"] = {}
        org_query = self.get_local_query("org-members.gql")

        # Retrieve members from member orgs.
        for org in self.settings.member_orgs:
            logger.info("\nRetrieving organization members for '%s'" % org)

            try:
                out = self.manager.queryGitHubFromFile(
                    org_query,
                    {"orgName": org, "numUsers": 50, "pgCursor": None},
                    paginate=True,
                    cursorVar="pgCursor",
                    keysToList=["data", "organization", "membersWithRole", "nodes"],
                )
            except Exception as error:
                print("Warning: Could not complete '%s': %s" % (org, error))
                continue

            # Update collective data
            for user in out["data"]["organization"]["membersWithRole"]["nodes"]:
                userKey = user["login"]
                self._data["internal-users"][user["login"]] = user

    def extract_external_users(self):
        """
        Find contributors across repos to label as external
        """
        users_query = self.get_local_query("repo-users.gql")

        # We will save internal and external users to file
        self._data["external-users"] = {}
        repos = self.load_dependency_file("repos")

        # Prepare repository names
        repolist = sorted(repos.data.keys())
        logger.info("Found %s associated repos parsed today." % len(repolist))

        # We need to already have the internal users
        if "internal-users" not in self._data:
            self.extract_internal_users()
        memberlist = sorted(self._data["internal-users"].keys())

        # Find contributors not associated with the org
        for repo in repolist:
            logger.info("Looking for contributors in %s" % repo)
            repo_user, repo_name = repo.split("/")

            try:
                out = self.manager.queryGitHubFromFile(
                    users_query,
                    {
                        "ownName": repo_user,
                        "repoName": repo_name,
                        "numUsers": 50,
                        "pgCursor": None,
                    },
                    paginate=True,
                    cursorVar="pgCursor",
                    keysToList=["data", "repository", "mentionableUsers", "nodes"],
                )
            except Exception as error:
                print("Warning: Could not complete '%s': %s" % (repo, error))
                continue

            # The users are found here!
            for user in out["data"]["repository"]["mentionableUsers"]["nodes"]:
                username = user["login"]
                if user["login"] in memberlist:
                    self.update_internal_user(user, repo)
                else:
                    self.update_external_user(user, repo)

    def update_internal_user(self, user, repo):
        return self._update_user_metadata(user, repo, "internal-users")

    def update_external_user(self, user, repo):
        return self._update_user_metadata(user, repo, "external-users")

    def _update_user_metadata(self, user, repo, key):
        """
        Given a username and a repository, update the data to include it.
        """
        username = user["login"]

        # External users will not be in list yet
        if username not in self._data[key]:
            self._data[key][username] = user

        if "contributedLabRepositories" not in self._data[key][username]:
            self._data[key][username]["contributedLabRepositories"] = {"nodes": []}
        self._data[key][username]["contributedLabRepositories"]["nodes"].append(repo)
        self._data[key][username]["contributedLabRepositories"]["nodes"].sort()



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

from contributor_ci.logger import logger


def main(args, parser, extra, subparser):

    from contributor_ci.main import Client

    cli = Client(
        quiet=args.quiet,
        config_file=args.config_file,
        outdir=args.outdir,
        save_format=args.save_format,
    )

    # Allowed extractors
    available = list(cli.extractors) + ["all"]
    selected = args.method or []
    for extractor in selected:
        if extractor not in available:
            logger.exit(
                "%s is not a valid extraction method! Choose from %s."
                % (extractor, ",".join(available))
            )

    # Case 1: a request for ALL
    if "all" in selected:
        cli.extract_all()
    else:
        for method in selected:
            cli.extract(method)



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

from contributor_ci.main.extractor import GitHubExtractorBase
from contributor_ci.logger import logger
import re
from datetime import datetime
from dateutil.relativedelta import relativedelta


class ActivityLines(GitHubExtractorBase):

    name = "activity_lines"
    description = "extract internal repository activity via lines of code."
    depends_on = ["repos"]
    filenames = ["activity_lines"]

    def extract(self):
        """
        Extract metadata about lines of code.
        """
        self._data[self.name] = {}

        # Load in dependency files
        repos = self.load_dependency_file("repos")
        repolist = sorted(repos.data.keys())

        # Code frequency endpoint
        query_in = "/repos/OWNNAME/REPONAME/stats/code_frequency"

        # Cutoff timestamp
        cutoff = int((datetime.now() - relativedelta(years=1)).timestamp())

        for repo in repolist:
            logger.info("Looking for lines of code in %s" % repo)
            repo_user, repo_name = repo.split("/")

            gitquery = re.sub("OWNNAME", repo_user, query_in)
            gitquery = re.sub("REPONAME", repo_name, gitquery)

            try:
                out = self.manager.queryGitHub(gitquery, rest=True)
            except Exception as error:
                logger.warning("Warning: Could not complete '%s': %s" % (repo, error))
                continue

            # Limit data to the past year
            out = list(filter(lambda x: x[0] > cutoff, out))

            for item in out:
                # Convert unix timestamps into standard dates (rounded to nearest week)
                weekinfo = datetime.utcfromtimestamp(item[0]).isocalendar()
                weekstring = str(weekinfo[0]) + "-W" + str(weekinfo[1]) + "-1"
                item[0] = datetime.strptime(weekstring, "%Y-W%W-%w").strftime(
                    "%Y-%m-%d"
                )

            self._data[self.name][repo] = out



#!/usr/bin/env python

__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

import contributor_ci
from contributor_ci.logger import setup_logger
import argparse
import sys
import os


def get_parser():
    parser = argparse.ArgumentParser(
        description="Contributor CI",
        formatter_class=argparse.RawTextHelpFormatter,
    )

    parser.add_argument(
        "--debug",
        dest="debug",
        help="use verbose logging to debug.",
        default=False,
        action="store_true",
    )

    parser.add_argument(
        "--quiet",
        dest="quiet",
        help="suppress additional output.",
        default=False,
        action="store_true",
    )

    parser.add_argument(
        "-c",
        "--config-file",
        dest="config_file",
        help="custom path to contributor-ci.yaml file.",
        default="contributor-ci.yaml",
    )

    parser.add_argument(
        "-o",
        "--out-dir",
        dest="outdir",
        help="Specify a custom output directory (defaults to $PWD/.cci)",
        default=None,
    )

    parser.add_argument(
        "--version",
        dest="version",
        help="suppress additional output.",
        default=False,
        action="store_true",
    )

    description = "actions for Contributor CI"
    subparsers = parser.add_subparsers(
        help="contributor-ci actions",
        title="actions",
        description=description,
        dest="command",
    )

    # print version and exit
    subparsers.add_parser("version", help="show software version")
    init = subparsers.add_parser("init", help="Create a contributor-ci.yaml.")
    init.add_argument("username", help="GitHub user or org name to base off of.")

    cfa = subparsers.add_parser("cfa", help="Contributor Friendliness Assessment")
    cfa.add_argument(
        "-t",
        "--terminal",
        action="store_true",
        default=False,
        help="print output to the terminal.",
    )
    cfa.add_argument(
        "repo", help="repository or 'all' to run for a contributor-ci.yaml file."
    )

    config = subparsers.add_parser(
        "config",
        description="update configuration file. Use sort, edit, add, or remove to edit fields.",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    config.add_argument(
        "params",
        nargs="*",
        help="edit, add, remove, or sort",
        type=str,
    )

    # See extractors
    listing = subparsers.add_parser(
        "list",
        description="see extractors available",
    )

    listing.add_argument(
        "query",
        help="search extractors by a query string",
        nargs="?",
        default=None,
    )
    # Local shell with client loaded
    shell = subparsers.add_parser(
        "shell", help="shell into a Python session with a client."
    )
    shell.add_argument("module_name", help="module to inspect", nargs="?")
    shell.add_argument(
        "--interpreter",
        "-i",
        dest="interpreter",
        help="python interpreter",
        choices=["ipython", "python", "bpython"],
        default="ipython",
    )

    # Extract metrics for a repository
    extract = subparsers.add_parser(
        "extract", description="extract metrics for a repository."
    )
    extract.add_argument("method", help="extraction method", nargs="*")
    extract.add_argument(
        "--save-format",
        dest="save_format",
        help="use a custom save format (e.g., year/month) defaults to year/month/day",
    )

    # Generate an interface with extractions, a GitHub workflow to run updates,
    # and contributor friendliness files.
    ui = subparsers.add_parser(
        "ui", description="generate or update a CCI user interface."
    )
    ui.add_argument("ui_command", help="generate or update.", nargs="*")
    ui.add_argument(
        "--cfa",
        dest="include_cfa",
        help="include contributor friendliness assessments for generate or update.",
        default=False,
        action="store_true",
    )

    return parser


def run():
    parser = get_parser()

    def help(return_code=0):
        """print help, including the software version and active client
        and exit with return code.
        """

        version = contributor_ci.__version__

        print("\nContributor CI v%s" % version)
        parser.print_help()
        sys.exit(return_code)

    # If the user didn't provide any arguments, show the full help
    if len(sys.argv) == 1:
        help()

    # If an error occurs while parsing the arguments, the interpreter will exit with value 2
    args, extra = parser.parse_known_args()

    if args.debug is True:
        os.environ["MESSAGELEVEL"] = "DEBUG"

    # Show the version and exit
    if args.command == "version" or args.version:
        print(contributor_ci.__version__)
        sys.exit(0)

    setup_logger(
        quiet=args.quiet,
        debug=args.debug,
    )

    # retrieve subparser (with help) from parser
    helper = None
    subparsers_actions = [
        action
        for action in parser._actions
        if isinstance(action, argparse._SubParsersAction)
    ]
    for subparsers_action in subparsers_actions:
        for choice, subparser in subparsers_action.choices.items():
            if choice == args.command:
                helper = subparser
                break

    if args.command == "cfa":
        from .cfa import main
    if args.command == "extract":
        from .extract import main
    if args.command == "init":
        from .init import main
    if args.command == "list":
        from .listing import main
    elif args.command == "shell":
        from .shell import main
    elif args.command == "config":
        from .config import main
    elif args.command == "ui":
        from .ui import main

    # Pass on to the correct parser
    return_code = 0
    try:
        main(args=args, parser=parser, extra=extra, subparser=helper)
        sys.exit(return_code)
    except UnboundLocalError:
        return_code = 1

    help(return_code)


if __name__ == "__main__":
    run()



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

from contributor_ci.main.extractor import GitHubExtractorBase
from contributor_ci.logger import logger


class MemberRepos(GitHubExtractorBase):

    name = "member_repos"
    description = "extract repositories that belong to members not within org."
    depends_on = ["users"]  # users already depends on repos
    filenames = ["member_repos"]

    def extract(self):
        """
        Extract metadata about member extra repositories.
        """
        self._data[self.name] = {}
        repo_query = self.get_local_query("user-repos.gql")

        # Load in dependency files
        repos = self.load_dependency_file("repos")
        users = self.load_dependency_file("internal-users")

        repolist = sorted(repos.data.keys())
        memberlist = sorted(users.data.keys())

        for user in memberlist:
            logger.info("Looking up extra repos for %s" % user)

            try:
                out = self.manager.queryGitHubFromFile(
                    repo_query,
                    {"userName": user, "numRepos": 50, "pgCursor": None},
                    paginate=True,
                    cursorVar="pgCursor",
                    keysToList=["data", "user", "repositoriesContributedTo", "nodes"],
                )
            except Exception as error:
                logger.warning("Warning: Could not complete '%s': %s" % (user, error))
                continue

            # Update collective data
            for repo in out["data"]["user"]["repositoriesContributedTo"]["nodes"]:
                key = repo["nameWithOwner"]

                # If the repository is an organization one, skip
                if key in repolist:
                    continue

                if key not in self._data[self.name]:
                    self._data[self.name][key] = repo
                self._data[self.name][key]["labContributors"] = {"nodes": []}
                self._data[self.name][key]["labContributors"]["nodes"].append(user)
                self._data[self.name][key]["labContributors"]["nodes"].sort()



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

from contributor_ci.logger import logger
from contributor_ci.main.extractor import GitHubExtractorBase

import os


class RepoDependencies(GitHubExtractorBase):

    name = "repo_dependencies"
    description = "extract repository dependencies."
    filenames = ["repo_dependencies"]
    depends_on = ["repos"]

    def extract(self):
        """
        Extract repository dependencies.
        """
        dep_query = self.get_local_query("repo-dependencies.gql")

        # This extract only saves one result - a repos file
        self._data[self.name] = {}

        # We require internal repositories list as input
        repos = self.load_dependency_file("repos")
        repolist = sorted(repos.data.keys())
        logger.info("Found %s repositories to find dependencies for." % len(repolist))

        for i, repo in enumerate(repolist):
            logger.info(
                "Getting dependencies for %s, %s of %s" % (repo, i, len(repolist))
            )
            repo_user, repo_name = repo.split("/")
            try:
                out = self.manager.queryGitHubFromFile(
                    dep_query,
                    {
                        "ownName": repo_user,
                        "repoName": repo_name,
                        "numManifests": 100,
                        "numDependents": 100,
                        "pgCursor": None,
                    },
                    paginate=True,
                    cursorVar="pgCursor",
                    keysToList=[
                        "data",
                        "repository",
                        "dependencyGraphManifests",
                        "nodes",
                    ],
                    headers={"Accept": "application/vnd.github.hawkgirl-preview+json"},
                )
            except Exception as error:
                logger.warning("Warning: Could not complete '%s':%s" % (repo, error))
                continue

            # save dependency data for repository
            self._data[self.name][repo] = out["data"]["repository"]



__author__ = "Vanessa Sochat"
__copyright__ = "Copyright 2021, Vanessa Sochat"
__license__ = "MPL 2.0"

from contributor_ci.logger import logger
import random
import os


def parse_option(options, option):
    """
    Parse an option into a name and value
    """
    # Currently supported options
    if not option.startswith("random"):
        logger.exit("%s is not a supported option." % option)
    if ":" in option:
        name, value = option.split(":", 1)
    else:
        name = option
        value = random.choice(range(2, 4))
    options[name] = int(value)
    return options


def main(args, parser, extra, subparser):

    from contributor_ci.main import Client

    cli = Client(quiet=args.quiet, config_file=args.config_file, outdir=args.outdir)

    command = args.ui_command.pop(0)

    # By default, generate or update in PWD if nothing provided
    dirname = os.getcwd()

    # We only accept a directory name for generate
    if command == "generate":
        if args.ui_command:
            dirname = args.ui_command.pop(0)
        cli.ui_generate(dirname, include_cfa=args.include_cfa)

    elif command == "update":

        # If we have another argument
        options = {}
        while args.ui_command:
            option = args.ui_command.pop(0)
            options = parse_option(options, option)

        # Assume directory is PWD
        cli.ui_update(dirname=".", options=options, include_cfa=args.include_cfa)
    else:
        logger.exit("%s is not a known user interface command." % command)
